unit dessin;

interface
uses classes,windows,graphics,utils;

const inf_egal='';
const sup_egal='';
const taille_point:integer=10;
const granularite:integer=100;
const couleur:tcolor=clblack;
const DEPART_GENERAL=-1;

type ptype_sommet=^type_sommet;
     type_sommet=record
     x,y:integer;
     libelle:string[2];
     end;

type ptableau=^ttableau;
    ttableau=array[0..51,0..51] of char;


type tnoeud=class
     function zoneoccupee:trect;virtual;
     procedure dessine(canvas:tcanvas);virtual;
     procedure translate(ax,ay:integer);virtual;
     function point_d_ancrage:tpoint;virtual;
     function point_de_depart(i:integer):tpoint;virtual;
     end;

type tsommet=class(tnoeud)
     public
     points:tlist;
     constructor create_point(ax,ay:integer;alibelle:string);
     constructor create_ensemble(liste:tlist);
     function ajoute_point(ax,ay:integer;alibelle:string):integer;
     function zoneoccupee:trect;override;
     procedure dessine(canvas:tcanvas);override;
     procedure translate(ax,ay:integer);override;
     function point_d_ancrage:tpoint;override;
     function point_de_depart(i:integer):tpoint;override;
     function position(libelle:string):integer;
     function element(i:integer):ptype_sommet;
     function nombre:integer;
     function max:integer;
     destructor free;
     property donneelement[i:integer]:ptype_sommet read element; default;
     end;

type ttrait=class(tnoeud)
     public
     sommet1,sommet2:tsommet;
     depart:integer;
     fleche:boolean;
     constructor create(asommet1,asommet2:tsommet;adepart:integer;afleche:boolean);
     function zoneoccupee:trect;override;
     procedure dessine(canvas:tcanvas);override;
     procedure translate(ax,ay:integer);override;
     function point_d_ancrage:tpoint;override;
     function point_de_depart(i:integer):tpoint;override;
     function reference(libelle:string):byte;
     function element(i:integer):tsommet;
     function nombre:integer;
     function max:integer;
     destructor free;
     property donneelement[i:integer]:tsommet read element;default;
     end;

type tgraphe_elementaire=class
     public
     traits:tlist;
     constructor create;
     function ajoutetrait(trait:ttrait):integer;
     function reference(libelle:string):integer;
     function zoneoccupee:trect;
     procedure dessine(canvas:tcanvas);
     procedure translate(ax,ay:integer);
     function element(i:integer):ttrait;
     function nombre:integer;
     function max:integer;
     destructor free;
     property donneelement[i:integer]:ttrait read element;default;
     end;

type grille=class
public
tableau:ptableau;
constructor create;
function litelement(etiquette1,etiquette2:string):char;
procedure fixeelement(etiquette1:string;etiquette2:string;valeur:char);
procedure construit_d_apres_chaine(chaine:string);
property element[etiquette1,etiquette2:string]:char read litelement write fixeelement;default;
end;

implementation

constructor grille.create;
begin
tableau:=new(ptableau);
fillchar(tableau^,sizeof(tableau),' ');
end;

procedure grille.construit_d_apres_chaine(chaine:string);
var
s,t:string;
u:char;
begin
while chaine<>'' do
      begin
      s:=copy(chaine,1,6);
      delete(chaine,1,4);
      t:=copy(s,3,2);
      u:=' ';
      if t[1]=t[2] then u:=t[1] else
      if t='<=' then u:=inf_egal else if t='>=' then u:=sup_egal;
      if u=' ' then continue;
      fixeelement(copy(s,1,2),copy(s,5,2),u);
      end;
end;



function grille.litelement(etiquette1,etiquette2:string):char;
var x,y:integer;
begin
result:=' ';
if length(etiquette1)<2 then exit;
if length(etiquette2)<2 then exit;
if (etiquette1[1]<'A') or (etiquette1[1]>'Z') or (etiquette2[1]<'A') or (etiquette2[1]>'Z')
or not(etiquette1[2] in ['+','-']) or not(etiquette2[2] in ['+','-']) then exit;
x:=(integer(etiquette1[1])-integer('A'))*2;
if etiquette1[2]='-' then inc(x);
y:=(integer(etiquette2[1])-integer('A'))*2;
if etiquette2[2]='-' then inc(y);
result:=tableau^[x,y];
end;

procedure grille.fixeelement(etiquette1:string;etiquette2:string;valeur:char);
var x,y:integer;
valeur2:char;

procedure propage(etiquette1:string;valeur:char;etiquette2:string);
var libelle:string[2];
i:integer;
begin
case valeur of
     ' ' : ;
     '?' : ;
     '=' :
         begin
         for i:=0 to 51 do
         begin
         libelle:=chr(byte('A')+(i div 2));
         if (i and 1)=0 then libelle:=libelle+'+' else libelle:=libelle+'-';
         case litelement(libelle,etiquette1) of
         '=' : fixeelement(libelle,etiquette2,'=');
         '<' : fixeelement(libelle,etiquette2,'<');
         '>' : fixeelement(libelle,etiquette2,'>');
         inf_egal : fixeelement(libelle,etiquette2,inf_egal);
         sup_egal : fixeelement(libelle,etiquette2,sup_egal);
         end;
         end;
         for i:=0 to 51 do
         begin
         libelle:=chr(byte('A')+(i div 2));
         if (i and 1)=0 then libelle:=libelle+'+' else libelle:=libelle+'-';
         case litelement(libelle,etiquette2) of
         '=' : fixeelement(libelle,etiquette1,'=');
         '<' : fixeelement(libelle,etiquette1,'<');
         '>' : fixeelement(libelle,etiquette1,'>');
         inf_egal : fixeelement(libelle,etiquette1,inf_egal);
         sup_egal : fixeelement(libelle,etiquette1,sup_egal);
         end;
         end;
         end;
     '<' :
         begin
         for i:=0 to 51 do
         begin
         libelle:=chr(byte('A')+(i div 2));
         if (i and 1)=0 then libelle:=libelle+'+' else libelle:=libelle+'-';
         case litelement(libelle,etiquette1) of
         '=','<',inf_egal : fixeelement(libelle,etiquette2,'<');
         end;
         end;
         for i:=0 to 51 do
         begin
         libelle:=chr(byte('A')+(i div 2));
         if (i and 1)=0 then libelle:=libelle+'+' else libelle:=libelle+'-';
         case litelement(libelle,etiquette2) of
         '=','>',sup_egal : fixeelement(libelle,etiquette1,'>');
         end;
         end;
         end;
     '>' :
         begin
         for i:=0 to 51 do
         begin
         libelle:=chr(byte('A')+(i div 2));
         if (i and 1)=0 then libelle:=libelle+'+' else libelle:=libelle+'-';
         case litelement(libelle,etiquette1) of
         '=','>',sup_egal : fixeelement(libelle,etiquette2,'>');
         end;
         end;
         for i:=0 to 51 do
         begin
         libelle:=chr(byte('A')+(i div 2));
         if (i and 1)=0 then libelle:=libelle+'+' else libelle:=libelle+'-';
         case litelement(libelle,etiquette2) of
         '=','<',inf_egal : fixeelement(libelle,etiquette1,'<');
         end;
         end;
         end;
     inf_egal :
         begin
         for i:=0 to 51 do
         begin
         libelle:=chr(byte('A')+(i div 2));
         if (i and 1)=0 then libelle:=libelle+'+' else libelle:=libelle+'-';
         case litelement(libelle,etiquette1) of
         '=',inf_egal : fixeelement(libelle,etiquette2,inf_egal);
         '<' : fixeelement(libelle,etiquette2,'<');
         end;
         end;
         for i:=0 to 51 do
         begin
         libelle:=chr(byte('A')+(i div 2));
         if (i and 1)=0 then libelle:=libelle+'+' else libelle:=libelle+'-';
         case litelement(libelle,etiquette2) of
         '=',sup_egal : fixeelement(libelle,etiquette1,sup_egal);
         '>' : fixeelement(libelle,etiquette1,'>');
         end;
         end;
         end;
     sup_egal :
              begin
         for i:=0 to 51 do
         begin
         libelle:=chr(byte('A')+(i div 2));
         if (i and 1)=0 then libelle:=libelle+'+' else libelle:=libelle+'-';
         case litelement(libelle,etiquette1) of
         '=',sup_egal : fixeelement(libelle,etiquette2,sup_egal);
         '>' : fixeelement(libelle,etiquette2,'>');
         end;
         end;
         for i:=0 to 51 do
         begin
         libelle:=chr(byte('A')+(i div 2));
         if (i and 1)=0 then libelle:=libelle+'+' else libelle:=libelle+'-';
         case litelement(libelle,etiquette2) of
         '=',inf_egal : fixeelement(libelle,etiquette1,inf_egal);
         '<' : fixeelement(libelle,etiquette1,'<');
         end;
         end;
         end;

         end;

end;


begin
if litelement(etiquette1,etiquette2)=valeur then exit;
if not(valeur in [' ','?','<','>','=',inf_egal,sup_egal,#0]) then exit;
if length(etiquette1)<2 then exit;
if length(etiquette2)<2 then exit;
if (etiquette1[1]<'A') or (etiquette1[1]>'Z') or (etiquette2[1]<'A') or (etiquette2[1]>'Z')
or not(etiquette1[2] in ['+','-']) or not(etiquette2[2] in ['+','-']) then exit;
x:=(integer(etiquette1[1])-integer('A'))*2;
if etiquette1[2]='-' then inc(x);
y:=(integer(etiquette2[1])-integer('A'))*2;
if etiquette2[2]='-' then inc(y);
tableau^[x,y]:=valeur;
if valeur=#0 then exit;
valeur2:=' ';
case valeur of
     ' ' : valeur2:=' ';
     '?' : valeur2:='?';
     '=' : valeur2:='=';
     '<' : valeur2:='>';
     '>' : valeur2:='<';
     inf_egal : valeur2:=sup_egal;
     sup_egal : valeur2:=inf_egal;
end;
if valeur=' ' then exit;
if litelement(etiquette2,etiquette1)=' ' then fixeelement(etiquette2,etiquette1,valeur2);
propage(etiquette1,valeur,etiquette2);
end;



constructor tgraphe_elementaire.create;
begin
inherited create;
traits:=tlist.Create;
end;


function tgraphe_elementaire.ajoutetrait(trait:ttrait):integer;
begin
result:=traits.add(trait);
end;

function tgraphe_elementaire.reference(libelle:string):integer;
var i:integer;
begin
result:=-1;
for i:=0 to traits.count-1 do if ttrait(traits.items[i]).reference(libelle)>0 then
    begin
    result:=i;
    exit;
    end;
end;

procedure tgraphe_elementaire.dessine(canvas:tcanvas);
var i:integer;
begin
for i:=0 to traits.count-1 do ttrait(traits.items[i]).dessine(canvas);
end;

function tgraphe_elementaire.zoneoccupee:trect;
var
x:trect;
i:integer;
begin
if traits.count=0 then
   begin
   result.top:=0;
   result.left:=0;
   result.right:=0;
   result.bottom:=0;
   end;
result:=ttrait(traits.items[0]).zoneoccupee;
for i:=1 to traits.count-1 do
    begin
    x:=ttrait(traits.items[i]).zoneoccupee;
    if result.top>x.top then result.top:=x.top;
    if result.left>x.left then result.left:=x.left;
    if result.right<x.right then result.right:=x.right;
    if result.bottom<x.bottom then result.bottom:=x.bottom;
    end;
end;

procedure tgraphe_elementaire.translate(ax,ay:integer);
var i:integer;
begin
for i:=0 to traits.count-1 do ttrait(traits.items[i]).translate(ax,ay);
end;

function tgraphe_elementaire.element(i:integer):ttrait;
begin
if i<traits.count then result:=traits.items[i] else result:=nil;
end;

function tgraphe_elementaire.nombre:integer;
begin
result:=traits.count;
end;

function tgraphe_elementaire.max:integer;
begin
result:=traits.count-1;
end;

destructor tgraphe_elementaire.free;
var i:integer;
begin
for i:=0 to traits.count-1 do ttrait(traits.items[i]).free;
traits.free;
inherited free;
end;


function tnoeud.zoneoccupee:trect;
begin
result.top:=0;
result.left:=0;
result.right:=0;
result.bottom:=0;
end;

procedure tnoeud.dessine(canvas:tcanvas);
begin
end;

procedure tnoeud.translate(ax,ay:integer);
begin
end;

function tnoeud.point_d_ancrage:tpoint;
begin
result.x:=0;
result.y:=0;
end;

function tnoeud.point_de_depart(i:integer):tpoint;
begin
result.x:=0;
result.y:=0;
end;





constructor tsommet.create_point(ax,ay:integer;alibelle:string);
var
x:ptype_sommet;
begin
inherited create;
new(x);
x.x:=ax;
x.y:=ay;
if length(alibelle)>0 then
      if (alibelle[1]>='a') and (alibelle[1]<='z') then alibelle[1]:=chr(byte(alibelle[1])-32);
x.libelle:=copy(alibelle,1,2);
points:=tlist.create;
points.add(x);
end;

constructor tsommet.create_ensemble(liste:tlist);
var i:integer;
begin
if points<>nil then
   for i:=0 to points.count -1 do
   dispose(ptype_sommet(points.items[i]));
points:=liste;
end;

function tsommet.ajoute_point(ax,ay:integer;alibelle:string):integer;
var
x:ptype_sommet;

begin
if points=nil then points:=tlist.create;
new(x);
x.x:=ax;
x.y:=ay;
x.libelle:=copy(alibelle,1,2);
result:=points.add(x);
end;


function tsommet.zoneoccupee:trect;
begin
result:=inherited zoneoccupee;
if (points<>nil) then if points.count>0 then
   begin
   result.top:=ptype_sommet(points.items[0])^.y-granularite+1;
   result.left:=ptype_sommet(points.items[0])^.x-granularite+1;
   result.bottom:=result.top+(points.Count-1)*(granularite div 2)+granularite-1;
   result.right:=result.left+granularite-1;
   end;

end;

procedure tsommet.dessine(canvas:tcanvas);
var i,x,y:integer;
libelle:string[1];
          begin
          if points=nil then exit;
          if points.count=0 then exit;
          with canvas do
               begin
               brush.style:=bssolid;
               brush.color:=couleur;
               font.color:=couleur;
               font.Name:='Arial';
               x:=ptype_sommet(points.items[0])^.x;
               y:=ptype_sommet(points.items[0])^.y;
               for i:=0 to points.count-1 do
                   begin
                   ellipse(x-taille_point,y-taille_point,x+taille_point,y+taille_point);
                   if taille_point>0 then font.size:=taille_point*2 else font.size:=4;
                   libelle:=copy(ptype_sommet(points.items[0])^.libelle,1,1);
                   textout(x-textwidth(libelle) div 2,y-taille_point-textheight(libelle),libelle);
                   y:=y+granularite div 2;
                   end;
               brush.style:=bsclear;
               if points.count>1 then ellipse(x-granularite div 2,ptype_sommet(points.items[0])^.y-granularite div 2,x+granularite div 2,y+granularite div 2);
               end;
          end;

procedure tsommet.translate(ax,ay:integer);
var i:integer;
begin
if points=nil then exit;
for i:=0 to points.count-1 do
    with ptype_sommet(points.items[i])^ do
         begin
         x:=x+ax;
         y:=y+ay;
         end;
end;

function tsommet.point_d_ancrage:tpoint;
var ycentre:integer;
begin
result:=inherited point_d_ancrage;
if points<>nil then
   begin
   if points.count=1 then
      begin
      result.y:=ptype_sommet(points.items[0])^.y;
      result.x:=ptype_sommet(points.items[0])^.x;
      end;
   if points.count>1 then
      begin
      result.x:=ptype_sommet(points.items[0])^.x-granularite div 2;
      ycentre:=ptype_sommet(points.items[0])^.y;
      ycentre:=ycentre*2;
      ycentre:=ycentre+points.count*(granularite div 2);
      result.y:=ycentre div 2;
      end;
   end;
end;

function tsommet.point_de_depart(i:integer):tpoint;
var ycentre:integer;
x,y:real;
begin
result:=inherited point_de_depart(i);
if points=nil then exit;
if (i>points.count) and (i<>DEPART_GENERAL) then exit;
ycentre:=ptype_sommet(points.items[0])^.y;
result.y:=ycentre;
result.x:=ptype_sommet(points.items[0])^.x;
if points.count=1 then exit;
if points.count>1 then
   begin
   ycentre:=ycentre*2;
   ycentre:=ycentre+(points.count-1)*(granularite div 2);
   ycentre:=ycentre div 2;
   end;
result.x:=ptype_sommet(points.items[0])^.x;
if i=DEPART_GENERAL then
   begin
   result.y:=ycentre;
   result.x:=result.x+granularite div 2;
   exit;
   end;
y:=ptype_sommet(points.items[0])^.y+i*(granularite div 2);
x:=(points.count+2);
x:=x*(granularite div 2);
x:=x*(ycentre-y);
x:=x*(ycentre-y);
x:=x/granularite;
x:=sqrt(x);
result.x:=result.x+round(x);
result.y:=round(y);
end;

function tsommet.position(libelle:string):integer;
var i:integer;
begin
libelle:=copy(libelle,1,2);
if length(libelle)>0 then
      if (libelle[1]>='a') and (libelle[1]<='z') then libelle[1]:=chr(byte(libelle[1])-32);
result:=-1;
if points=nil then exit;
for i:=0 to points.count-1 do
    if libelle=ptype_sommet(points.items[i]).libelle then
       begin
       result:=i;
       exit;
       end;
end;

function tsommet.element(i:integer):ptype_sommet;
begin
result:=nil;
if points=nil then exit;
if i>=points.count then exit;
result:=points.items[i];
end;

function tsommet.nombre:integer;
begin
result:=points.count;
end;

function tsommet.max:integer;
begin
result:=points.count-1;
end;

destructor tsommet.free;
var i:integer;
begin
if points<>nil then
   for i:=0 to points.count -1 do
   dispose(ptype_sommet(points.items[i]));
inherited free;
end;

constructor ttrait.create(asommet1,asommet2:tsommet;adepart:integer;afleche:boolean);
begin
inherited create;
sommet1:=asommet1;
sommet2:=asommet2;
depart:=adepart;
fleche:=afleche;
end;

function ttrait.zoneoccupee:trect;
var
x:trect;
begin
result:=sommet1.zoneoccupee;
x:=sommet2.zoneoccupee;
if result.top>x.top then result.top:=x.top;
if result.left>x.left then result.left:=x.left;
if result.right<x.right then result.right:=x.right;
if result.bottom<x.bottom then result.bottom:=x.bottom;
end;

procedure ttrait.dessine(canvas:tcanvas);
var
point1,point2:tpoint;
begin
if (sommet1=nil) or (sommet2=nil) then exit;
with canvas do
     begin
     pen.color:=couleur;
     pen.style:=pssolid;
     Pen.mode:=pmcopy;
     brush.color:=couleur;
     brush.style:=bssolid;
     point1:=sommet1.point_de_depart(depart);
     point2:=sommet2.point_d_ancrage;
     if not fleche then
        begin
        moveto(point1.x,point1.y);
        lineto(point2.x,point2.y);
        end
     else dessinefleche(canvas,point1.x,point1.y,point2.x,point2.y,taille_point*3);
     end;
end;

procedure ttrait.translate(ax,ay:integer);
begin
if (sommet1<>nil) then sommet1.translate(ax,ay);
if (sommet2<>nil) then sommet2.translate(ax,ay);
end;

function ttrait.point_d_ancrage:tpoint;
begin
result:=inherited point_d_ancrage;
if (sommet1)<>nil then result:=sommet1.point_d_ancrage;
end;

function ttrait.point_de_depart(i:integer):tpoint;
begin
result:=inherited point_de_depart(i);
if (sommet2)<>nil then result:=sommet2.point_de_depart(i);
end;

function ttrait.reference(libelle:string):byte;
begin
result:=0;
if sommet1<>nil then if sommet1.position(libelle)<>-1 then result:=result+1;
if sommet2<>nil then if sommet2.position(libelle)<>-1 then result:=result+2;
end;

function ttrait.element(i:integer):tsommet;
begin
result:=nil;
if i=0 then if sommet1<>nil then result:=sommet1 else result:=sommet2;
if i=1 then if sommet1<>nil then result:=sommet2;
end;

function ttrait.nombre:integer;
begin
result:=0;
if sommet1<>nil then result:=result+1;
if sommet2<>nil then result:=result+1;
end;

function ttrait.max:integer;
begin
result:=nombre-1;
end;

destructor ttrait.free;
begin
sommet1.free;
sommet2.free;
inherited free;
end;


end.
