unit graphes;

interface
uses classes,windows,graphics,utils,dialogs,sysutils;

const inf_egal='';
const sup_egal='';
const taille_point:integer=10;
const granularite:integer=100;
const couleur:tcolor=clblack;
const DEPART_GENERAL=-1;
const sortie_ecran:boolean=true;
const fond_blanc:boolean=true;

type ptype_sommet=^type_sommet;
     type_sommet=record
     x,y:integer;
     libelle:string[2];
     end;

type ptableau=^ttableau;
    ttableau=array[0..51,0..51] of char;


type tnoeud=class
     function zoneoccupee:trect;virtual;
     procedure dessine(canvas:tcanvas);virtual;
     procedure translate(ax,ay:integer);virtual;
     function point_d_ancrage:tpoint;virtual;
     function point_de_depart(i:integer):tpoint;virtual;
     end;

type tsommet=class(tnoeud)
     public
     pos:integer;
     points:tlist;
     constructor create_point(ax,ay:integer;alibelle:string);
     constructor create_ensemble(liste:tlist);
     function ajoute_point(ax,ay:integer;alibelle:string):integer;
     function retire_point(position:integer):integer;
     function zoneoccupee:trect;override;
     procedure dessine(canvas:tcanvas);override;
     procedure translate(ax,ay:integer);override;
     function point_d_ancrage:tpoint;override;
     function point_de_depart(i:integer):tpoint;override;
     function position(libelle:string):integer;
     function element(i:integer):ptype_sommet;
     function nombre:integer;
     function max:integer;
     function etiquettes:string;
     function hauteur:integer;
     destructor free;
     property donneelement[i:integer]:ptype_sommet read element; default;
     end;

type ttrait=class(tnoeud)
     public
     tag:integer;
     sommet1,sommet2:tsommet;
     depart:integer;
     fleche:boolean;
     constructor create(asommet1,asommet2:tsommet;adepart:integer;afleche:boolean);
     function zoneoccupee:trect;override;
     procedure dessine(canvas:tcanvas);override;
     procedure translate(ax,ay:integer);override;
     function point_d_ancrage:tpoint;override;
     function point_de_depart(i:integer):tpoint;override;
     function reference(libelle:string):byte;
     function element(i:integer):tsommet;
     function nombre:integer;
     function max:integer;
     destructor free;
     property donneelement[i:integer]:tsommet read element;default;
     end;

type tgraphe_elementaire=class
     public
     traits:tlist;
     constructor create;
     function ajoutetrait(trait:ttrait):integer;
     function reference(libelle:string):integer;
     function zoneoccupee:trect;
     procedure dessine(canvas:tcanvas);
     procedure translate(ax,ay:integer);
     function element(i:integer):ttrait;
     function nombre:integer;
     function max:integer;
     destructor free;
     property donneelement[i:integer]:ttrait read element;default;
     end;

type tgrille=class
public
source:string;
erreurs:string;
tableau:ptableau;
domaine:array[0..25] of boolean;
restrictions:array[0..25] of boolean;
constructor create;
procedure initialise;
function litelement(etiquette1,etiquette2:string):char;
procedure fixeelement(etiquette1:string;etiquette2:string;valeur:char);
procedure construit_d_apres_chaine(chaine:string);
property element[etiquette1,etiquette2:string]:char read litelement write fixeelement;default;
destructor free;
end;


type tgraphe=class
     public
     liste_sommets:tlist;
     traits:tlist;
     equivalent_ponctuel:string;
     constructor create;
     procedure initialise_d_apres_grille(grille:tgrille);
     destructor free;
     function ajoutetrait(trait:ttrait):integer;
     procedure dessine(canvas:tcanvas);
     procedure translate(ax,ay:integer);
     function zoneoccupee:trect;
     function element(i:integer):ttrait;
     function nombre:integer;
     function max:integer;
     function element_sommet(i:integer):tsommet;
     function nombre_sommets:integer;
     function max_sommets:integer;
     function cherchesommet(libelle:string;var position:integer):tsommet;
     function chemin(x,y:tsommet):boolean;
     function chemin_indirect(x,y:tsommet):boolean;
     function equiv_ponctuel(x:tsommet):string;
     procedure reinitialise;
     property donneelement[i:integer]:ttrait read element;default;
     property sommets[i:integer]:tsommet read element_sommet;
     end;



implementation

var rect1,rect2:trect;



constructor tgraphe.create;
begin
inherited create;
liste_sommets:=tlist.create;
traits:=tlist.create;
end;

function tgraphe.chemin(x,y:tsommet):boolean;
var i:integer;
begin
for i:=0 to traits.count-1 do
    if ttrait(traits.items[i]).sommet1=x then
       begin
       if ttrait(traits.items[i]).sommet2=y then
          begin
          result:=true;
          exit;
          end else
              if chemin(ttrait(traits.items[i]).sommet2,y) then
                 begin
                 result:=true;
                 exit;
                 end;
        end;
result:=false;
end;

function tgraphe.chemin_indirect(x,y:tsommet):boolean;
var i:integer;
begin
for i:=0 to traits.count-1 do
    if ttrait(traits.items[i]).sommet1=x then
       begin
       if element(i).sommet2<>y then
              if chemin(ttrait(traits.items[i]).sommet2,y) then
                 begin
                 result:=true;
                 exit;
                 end;
       end;
result:=false;
end;

function tgraphe.equiv_ponctuel(x:tsommet):string;
var i:integer;
compteur:integer;
s:string;
begin
compteur:=0;

s:=x.etiquettes;
if x.pos<>-1 then begin result:=copy(s,1,2); exit;end;

result:='';

if length(s)=2 then result:=s else
   begin
   while length(s)>=4 do
         begin
         result:=result+copy(s,1,2)+'==';
         delete(s,1,2);
         end;
   if length(s)=2 then result:=result+s;
   end;
for i:=0 to traits.count-1 do
    if ttrait(traits.items[i]).sommet1=x then
    if ttrait(traits.items[i]).tag=-1 then
       begin
       ttrait(traits.items[i]).tag:=0;
       s:=equiv_ponctuel(ttrait(traits.items[i]).sommet2);
       if compteur>0 then result:=result+'  '+copy(x.etiquettes,length(x.etiquettes)-1,2);
       if ttrait(traits.items[i]).fleche then result:=result+'<=' else result:=result+'<<';
//       s:=copy(ttrait(traits.items[i]).sommet2.etiquettes,1,2);
       result:=result+s;
       inc(compteur);
       end;
x.pos:=0;

end;





procedure tgraphe.initialise_d_apres_grille(grille:tgrille);
var i,j,k,l,m:integer;
chaine,s,t:string;
trouve:boolean;
x:tsommet;
position:integer;
v:char;
sommet1,sommet2:tsommet;
position1,position2:integer;
trait:ttrait;
graphe_elementaire:tgraphe_elementaire;
recommencer,recommencer2:boolean;
bidon,bidon2:ttrait;

function trouve_position(i:integer):integer;
var j,max:integer;
s,t:string;
u:integer;
begin
if sommets[i].pos<>-1 then
   begin
   result:=sommets[i].pos;
   exit;
   end;
s:=copy(sommets[i].etiquettes,1,2);
max:=-1;
for j:=0 to max_sommets do
    begin
    t:=copy(sommets[j].etiquettes,1,2);
    if grille[s,t] in ['>',sup_egal] then
       begin
       u:=trouve_position(j);
       if u>max then max:=u;
       end;
    end;
if max=-1 then result:=1 else result:=max+1;
end;

function trouve_positiony(i:tsommet):integer;
var j:integer;
x1,x2,v,y:integer;
recommencer:boolean;
w:tsommet;
begin
if i.pos<>-1 then
   begin
   result:=i.pos;
   exit;
   end;
x1:=-1000000;
x2:=1000000;
y:=0;
      for j:=0 to max do
          if element(j).sommet2.etiquettes=i.etiquettes then
             begin
             w:=element(j).sommet1;
             v:=trouve_positiony(w);
//             v:=w.pos;
             v:=v+w.point_d_ancrage.y;
             if v>x1 then x1:=v;
             if v<x2 then x2:=v;
             y:=y+1;
             end;
if y=0 then result:=granularite div 2-i.point_d_ancrage.y else result:=(x1+x2) div 2-i.point_d_ancrage.y;
i.pos:=result;
repeat
recommencer:=false;
for j:=0 to max_sommets do if sommets[j].pos<>-1 then if sommets[j].etiquettes<>i.etiquettes then
    begin
    rect1:=sommets[j].zoneoccupee;
    rect2:=i.zoneoccupee;
    if (rect2.left>rect1.right) or (rect2.right<rect1.left) then continue;
    rect1.top:=rect1.top+sommets[j].pos;
    rect1.bottom:=rect1.bottom+sommets[j].pos;
    rect2.top:=rect2.top+i.pos;
    rect2.bottom:=rect2.bottom+i.pos;
    while ((rect2.top>=rect1.top) and (rect2.top<=rect1.bottom)) or
          ((rect2.top<=rect1.top) and (rect2.bottom<=rect1.bottom)) do
          begin
          i.pos:=i.pos+granularite div 10;
          rect2.top:=rect2.top+granularite div 10;
          rect2.bottom:=rect2.bottom+ granularite div 10;
          recommencer:=true;
          end;
    end;
until not recommencer;
result:=i.pos;
end;



begin
// Cration des sommets
for i:=0 to 25 do if grille.domaine[i] then
    for j:=0 to 1
        do
        begin
        s:=chr(integer('A')+i);
        if j=0 then s:=s+'-' else s:=s+'+';
        trouve:=false;
        for k:=0 to max_sommets do
            begin
            t:=sommets[k].etiquettes;
            for l:=1 to (length(t) div 2) do
                if grille[s,copy(t,l*2-1,2)]='=' then
                   begin
                   sommets[k].ajoute_point(0,0,s);
                   trouve:=true;
                   break;
                   end;
            if trouve then break;
            end;
        if trouve then continue;
        x:=tsommet.create_point(0,0,s);
        liste_sommets.add(x);
        end;
// positionnement horizontal des sommets
for i:=0 to max_sommets do sommets[i].pos:=trouve_position(i);
for i:=0 to max_sommets do with sommets[i] do
         begin
         translate(pos*granularite,0);
         pos:=-1;
         end;
// positionnement des traits
chaine:=grille.source;
while length(chaine)>=6 do
      begin
      s:=copy(chaine,1,6);
      delete(chaine,1,4);
      t:=copy(s,3,2);
      v:=' ';
      if t[1]=t[2] then v:=t[1] else
      if t='<=' then v:=inf_egal else if t='>=' then v:=sup_egal;
      if (v=' ') or (v='=') then continue;
      sommet1:=cherchesommet(copy(s,1,2),position1);
      sommet2:=cherchesommet(copy(s,5,2),position2);
      {if position1<>0 then} position1:=DEPART_GENERAL;
      trait:=ttrait.create(sommet1,sommet2,position1,v=inf_egal);
      ajoutetrait(trait);
      end;
//suppression des traits redondants
repeat
recommencer:=false;
for i:=0 to max do
    begin
    if chemin_indirect(ttrait(traits.items[i]).sommet1,ttrait(traits.items[i]).sommet2) then
       begin
       bidon:=ttrait(traits.items[i]);
       bidon.free;
       traits.delete(i);
       recommencer:=true;
       break;
       end;
    end;
until not recommencer;
//suppression des traits identiques
repeat
recommencer:=false;
for i:=0 to max do
    begin
    bidon:=traits.items[i];
    for j:=0 to max do
    begin
    bidon2:=traits.items[j];
    if bidon<>bidon2 then
    if (bidon.sommet1=bidon2.sommet1) and (bidon.sommet2=bidon2.sommet2) and (bidon.fleche=bidon2.fleche) then
              begin
              bidon.fleche:=bidon.fleche and bidon2.fleche;
              bidon2.free;
              traits.delete(j);
              recommencer:=true;
              break;
              end;
    end;
    if recommencer then break;
    end;
    until not recommencer;
//suppression des sommets choisis par menu
for i:=0 to 25 do if grille.restrictions[i] then
    for k:=0 to 1 do
        begin
        s:=chr(integer('A')+i);
        if k=0 then s:=s+'-' else s:=s+'+';
        repeat
        recommencer2:=false;
        for j:=0 to max_sommets do
            begin
            position1:=sommets[j].position(s);
            if position1<>-1 then
               begin
               sommets[j].retire_point(position1);
               if sommets[j].nombre=0 then
                  begin
                  for l:=0 to traits.count-1 do
                      if element(l).sommet1=sommets[j] then
                         begin
                         for m:=0 to traits.count-1 do
                             if element(m).sommet2=sommets[j] then
                                begin
                                bidon:=ttrait.create(element(m).sommet1,element(l).sommet2,element(m).depart,element(l).fleche and element(m).fleche);
                                ajoutetrait(bidon);
                                end;
                         end;
                  for l:=0 to traits.count-1 do
                      if element(l).sommet2=sommets[j] then
                         begin
                         for m:=0 to traits.count-1 do
                             if element(m).sommet1=sommets[j] then
                                begin
                                bidon:=ttrait.create(element(m).sommet1,element(l).sommet2,depart_general,element(l).fleche and element(m).fleche);
                                ajoutetrait(bidon);
                                end;
                          end;
                  repeat
                  recommencer:=false;
                  l:=0;
                  while l<traits.count do
                      begin
                      if (element(l).sommet1=sommets[j]) or
                         (element(l).sommet2=sommets[j]) then
                         begin
                         recommencer:=true;
                         bidon:=ttrait(traits.items[l]);
                         bidon.free;
                         traits.delete(l);
                         end;
                       inc(l);
                       end;
                  until not recommencer;
                  tsommet(liste_sommets.items[j]).free;
                  liste_sommets.delete(j);
                  recommencer2:=true;
                  break;
                  end;
               end;
            end;
            until not recommencer2;
        end;
//re-suppression des traits redondants
repeat
recommencer:=false;
for i:=0 to max do
    begin
    if chemin_indirect(ttrait(traits.items[i]).sommet1,ttrait(traits.items[i]).sommet2) then
       begin
       bidon:=ttrait(traits.items[i]);
       bidon.free;
       traits.delete(i);
       recommencer:=true;
       break;
       end;
    end;
until not recommencer;
//re-suppression des traits identiques
repeat
recommencer:=false;
for i:=0 to max do
    begin
    bidon:=traits.items[i];
    for j:=0 to max do
    begin
    bidon2:=traits.items[j];
    if bidon<>bidon2 then
    if (bidon.sommet1=bidon2.sommet1) and (bidon.sommet2=bidon2.sommet2) and (bidon.fleche=bidon2.fleche) then
              begin
              bidon.fleche:=bidon.fleche and bidon2.fleche;
              bidon2.free;
              traits.delete(j);
              recommencer:=true;
              break;
              end;
    end;
    if recommencer then break;
    end;
    until not recommencer;
//placement vertical des sommets
for i:=0 to max_sommets do sommets[i].pos:=trouve_positiony(sommets[i]);
position1:=0;
for i:=0 to max_sommets do with sommets[i] do
         begin
         if pos-granularite<position1 then position1:=pos-granularite;
         translate(0,pos);
         pos:=-1;
         end;
if position1<>0 then
for i:=0 to max_sommets do with sommets[i] do
         begin
         if pos-granularite<position1 then position1:=pos-granularite;
         translate(0,-position1);
         end;
//calcul de l'quivalent ponctuel
for i:=0 to max do ttrait(traits.items[i]).tag:=-1;
position1:=-1;
equivalent_ponctuel:='';
for i:=0 to max_sommets do
    begin
    position2:=ptype_sommet(sommets[i][0]).x div granularite;
    if position2>position1 then position1:=position2;
    end;
position1:=1;
while position1<=position2 do
      begin
      for i:=0 to max_sommets do
          if sommets[i].pos=-1 then
          if ptype_sommet(sommets[i][0]).x div granularite=position1 then
          begin
          if equivalent_ponctuel<>'' then equivalent_ponctuel:=equivalent_ponctuel+'  ';
          equivalent_ponctuel:=equivalent_ponctuel+equiv_ponctuel(sommets[i]);
          end;
      inc(position1);
      end;

end;

function tgraphe.cherchesommet(libelle:string;var position:integer):tsommet;
var i,j:integer;
begin
for i:=0 to max_sommets do
    begin
    j:=pos(libelle,sommets[i].etiquettes);
    if j=0 then continue;
    position:=(j-1) div 2;
    result:=sommets[i];
    exit;
    end;
position:=-1;
result:=nil;
end;

function tgraphe.ajoutetrait(trait:ttrait):integer;
begin
result:=traits.add(trait);
end;


procedure tgraphe.dessine(canvas:tcanvas);
var i:integer;
begin
for i:=0 to traits.count-1 do ttrait(traits.items[i]).dessine(canvas);
for i:=0 to max_sommets do sommets[i].dessine(canvas);
end;

procedure tgraphe.translate(ax,ay:integer);
var i:integer;
begin
for i:=0 to traits.count-1 do ttrait(traits.items[i]).translate(ax,ay);
end;

function tgraphe.zoneoccupee:trect;
var
x:trect;
i:integer;
begin
if traits.count=0 then
   begin
   result.top:=0;
   result.left:=0;
   result.right:=0;
   result.bottom:=0;
   end;
if traits.count=0 then exit;
result:=ttrait(traits.items[0]).zoneoccupee;
for i:=1 to traits.count-1 do
    begin
    x:=ttrait(traits.items[i]).zoneoccupee;
    if result.top>x.top then result.top:=x.top;
    if result.left>x.left then result.left:=x.left;
    if result.right<x.right then result.right:=x.right;
    if result.bottom<x.bottom then result.bottom:=x.bottom;
    end;
end;

function tgraphe.element(i:integer):ttrait;
begin
if i<traits.count then result:=traits.items[i] else result:=nil;
end;

function tgraphe.nombre:integer;
begin
result:=traits.count;
end;

function tgraphe.max:integer;
begin
result:=traits.count-1;
end;

function tgraphe.element_sommet(i:integer):tsommet;
begin
if i<liste_sommets.count then result:=liste_sommets.items[i] else result:=nil;
end;

function tgraphe.nombre_sommets:integer;
begin
result:=liste_sommets.count;
end;

function tgraphe.max_sommets:integer;
begin
result:=liste_sommets.count-1;
end;

procedure tgraphe.reinitialise;
var i:integer;
x:tsommet;
begin
while liste_sommets.count>0 do
    begin
    if liste_sommets.items[0]<>nil then
       tsommet(liste_sommets.items[0]).free;
    liste_sommets.delete(0);
    end;
while  traits.count>0 do
    begin
    if traits.items[0]<>nil then
       ttrait(traits.items[0]).free;
    traits.Delete(0);
    end;
end;

destructor tgraphe.free;
var i:integer;
begin
reinitialise;
liste_sommets.free;
traits.free;
inherited free;
end;

constructor tgrille.create;
begin
tableau:=new(ptableau);
initialise;
end;

procedure tgrille.initialise;
var i:integer;
begin
fillchar(tableau^,sizeof(tableau^),' ');
for i:=byte('A') to byte('Z') do fixeelement(chr(i)+'-',chr(i)+'+','<');
fillchar(domaine,sizeof(domaine),0);
fillchar(restrictions,sizeof(domaine),0);
source:='';
erreurs:='';
end;

procedure tgrille.construit_d_apres_chaine(chaine:string);
var
s,t:string;
u:char;
begin
if source<>'' then source:=source+'  ';
source:=source+chaine;
while length(chaine)>=6 do
      begin
      s:=copy(chaine,1,6);
      delete(chaine,1,4);
      t:=copy(s,3,2);
      u:=' ';
      if t[1]=t[2] then u:=t[1] else
      if t='<=' then u:=inf_egal else if t='>=' then u:=sup_egal;
      if u=' ' then continue;
      fixeelement(copy(s,1,2),copy(s,5,2),u);
      end;
end;

function tgrille.litelement(etiquette1,etiquette2:string):char;
var x,y:integer;
begin
result:=' ';
if length(etiquette1)<2 then exit;
if length(etiquette2)<2 then exit;
if (etiquette1[1]<'A') or (etiquette1[1]>'Z') or (etiquette2[1]<'A') or (etiquette2[1]>'Z')
or not(etiquette1[2] in ['+','-']) or not(etiquette2[2] in ['+','-']) then exit;
x:=(integer(etiquette1[1])-integer('A'))*2;
if etiquette1[2]='-' then inc(x);
y:=(integer(etiquette2[1])-integer('A'))*2;
if etiquette2[2]='-' then inc(y);
result:=tableau^[x,y];
end;

procedure tgrille.fixeelement(etiquette1:string;etiquette2:string;valeur:char);
var x,y:integer;
valeur2:char;

procedure propage(etiquette1:string;valeur:char;etiquette2:string);
var libelle:string[2];
i:integer;
begin
case valeur of
     ' ' : ;
     '?' : ;
     '=' :
         begin
         for i:=0 to 51 do
         begin
         libelle:=chr(byte('A')+(i div 2));
         if (i and 1)=0 then libelle:=libelle+'+' else libelle:=libelle+'-';
         case litelement(libelle,etiquette1) of
         '=' : if libelle<>etiquette2 then fixeelement(libelle,etiquette2,'=');
         '<' : fixeelement(libelle,etiquette2,'<');
         '>' : fixeelement(libelle,etiquette2,'>');
         inf_egal : fixeelement(libelle,etiquette2,inf_egal);
         sup_egal : fixeelement(libelle,etiquette2,sup_egal);
         end;
         end;
         for i:=0 to 51 do
         begin
         libelle:=chr(byte('A')+(i div 2));
         if (i and 1)=0 then libelle:=libelle+'+' else libelle:=libelle+'-';
         case litelement(libelle,etiquette2) of
         '=' : if libelle<>etiquette1 then fixeelement(libelle,etiquette1,'=');
         '<' : fixeelement(libelle,etiquette1,'<');
         '>' : fixeelement(libelle,etiquette1,'>');
         inf_egal : fixeelement(libelle,etiquette1,inf_egal);
         sup_egal : fixeelement(libelle,etiquette1,sup_egal);
         end;
         end;
         end;
     '<' :
         begin
         for i:=0 to 51 do
         begin
         libelle:=chr(byte('A')+(i div 2));
         if (i and 1)=0 then libelle:=libelle+'+' else libelle:=libelle+'-';
         case litelement(libelle,etiquette1) of
         '=','<',inf_egal : fixeelement(libelle,etiquette2,'<');
         end;
         end;
         for i:=0 to 51 do
         begin
         libelle:=chr(byte('A')+(i div 2));
         if (i and 1)=0 then libelle:=libelle+'+' else libelle:=libelle+'-';
         case litelement(libelle,etiquette2) of
         '=','>',sup_egal : fixeelement(libelle,etiquette1,'>');
         end;
         end;
         end;
     '>' :
         begin
         for i:=0 to 51 do
         begin
         libelle:=chr(byte('A')+(i div 2));
         if (i and 1)=0 then libelle:=libelle+'+' else libelle:=libelle+'-';
         case litelement(libelle,etiquette1) of
         '=','>',sup_egal : fixeelement(libelle,etiquette2,'>');
         end;
         end;
         for i:=0 to 51 do
         begin
         libelle:=chr(byte('A')+(i div 2));
         if (i and 1)=0 then libelle:=libelle+'+' else libelle:=libelle+'-';
         case litelement(libelle,etiquette2) of
         '=','<',inf_egal : fixeelement(libelle,etiquette1,'<');
         end;
         end;
         end;
     inf_egal :
         begin
         for i:=0 to 51 do
         begin
         libelle:=chr(byte('A')+(i div 2));
         if (i and 1)=0 then libelle:=libelle+'+' else libelle:=libelle+'-';
         case litelement(libelle,etiquette1) of
         '=',inf_egal : fixeelement(libelle,etiquette2,inf_egal);
         '<' : fixeelement(libelle,etiquette2,'<');
         end;
         end;
         for i:=0 to 51 do
         begin
         libelle:=chr(byte('A')+(i div 2));
         if (i and 1)=0 then libelle:=libelle+'+' else libelle:=libelle+'-';
         case litelement(libelle,etiquette2) of
         '=',sup_egal : fixeelement(libelle,etiquette1,sup_egal);
         '>' : fixeelement(libelle,etiquette1,'>');
         end;
         end;
         end;
     sup_egal :
         begin
         for i:=0 to 51 do
         begin
         libelle:=chr(byte('A')+(i div 2));
         if (i and 1)=0 then libelle:=libelle+'+' else libelle:=libelle+'-';
         case litelement(libelle,etiquette1) of
         '=',sup_egal : fixeelement(libelle,etiquette2,sup_egal);
         '>' : fixeelement(libelle,etiquette2,'>');
         end;
         end;
         for i:=0 to 51 do
         begin
         libelle:=chr(byte('A')+(i div 2));
         if (i and 1)=0 then libelle:=libelle+'+' else libelle:=libelle+'-';
         case litelement(libelle,etiquette2) of
         '=',inf_egal : fixeelement(libelle,etiquette1,inf_egal);
         '<' : fixeelement(libelle,etiquette1,'<');
         end;
         end;
         end;
         end;
end;


procedure memoriseerreur;
begin
erreurs:=erreurs+etiquette1+etiquette2+valeur+valeur2;
end;

begin
if not(valeur in [' ','?','<','>','=',inf_egal,sup_egal,#0]) then exit;
if length(etiquette1)<2 then exit;
if length(etiquette2)<2 then exit;
if (etiquette1[1]<'A') or (etiquette1[1]>'Z') or (etiquette2[1]<'A') or (etiquette2[1]>'Z')
or not(etiquette1[2] in ['+','-']) or not(etiquette2[2] in ['+','-']) then exit;
domaine[integer(etiquette1[1])-integer('A')]:=true;
domaine[integer(etiquette2[1])-integer('A')]:=true;
if (etiquette1[1]=etiquette2[1]) then
   begin
   if etiquette1[2]=etiquette2[2] then exit;
   if (etiquette1[2]='-') and (etiquette2[2]='+') {then if valeur=inf_egal} then valeur:='<';
   if (etiquette1[2]='+') and (etiquette2[2]='-') {then if valeur=sup_egal} then valeur:='>';
   end;
valeur2:=litelement(etiquette1,etiquette2);
if valeur2=valeur then exit;
case valeur of
     ' ' : ;
     '?' : ;
     '=' : if valeur2 in ['<','>',inf_egal,sup_egal] then memoriseerreur;
     '<' : if valeur2 in ['=','>',inf_egal,sup_egal] then memoriseerreur;
     '>' : if valeur2 in ['=','<',inf_egal,sup_egal] then memoriseerreur;
     inf_egal :
               begin
               if valeur2='<' then valeur:='<';
               if valeur2 in ['=','<','>',sup_egal] then memoriseerreur;
               end;
     sup_egal :    begin
                   if valeur2='>' then valeur:='>';
                   if valeur2 in ['=','<','>',inf_egal] then memoriseerreur;
                   end;
end;
x:=(integer(etiquette1[1])-integer('A'))*2;
if etiquette1[2]='-' then inc(x);
y:=(integer(etiquette2[1])-integer('A'))*2;
if etiquette2[2]='-' then inc(y);
tableau^[x,y]:=valeur;
if valeur=#0 then exit;
valeur2:=' ';
case valeur of
     ' ' : valeur2:=' ';
     '?' : valeur2:='?';
     '=' : valeur2:='=';
     '<' : valeur2:='>';
     '>' : valeur2:='<';
     inf_egal : valeur2:=sup_egal;
     sup_egal : valeur2:=inf_egal;
end;
if valeur=' ' then exit;
if litelement(etiquette2,etiquette1)=' ' then fixeelement(etiquette2,etiquette1,valeur2);
propage(etiquette1,valeur,etiquette2);
end;

destructor tgrille.free;
begin
dispose(tableau);
inherited free;
end;

constructor tgraphe_elementaire.create;
begin
inherited create;
traits:=tlist.Create;
end;


function tgraphe_elementaire.ajoutetrait(trait:ttrait):integer;
begin
result:=traits.add(trait);
end;

function tgraphe_elementaire.reference(libelle:string):integer;
var i:integer;
begin
result:=-1;
for i:=0 to traits.count-1 do if ttrait(traits.items[i]).reference(libelle)>0 then
    begin
    result:=i;
    exit;
    end;
end;

procedure tgraphe_elementaire.dessine(canvas:tcanvas);
var i:integer;
begin
for i:=0 to traits.count-1 do ttrait(traits.items[i]).dessine(canvas);
end;

function tgraphe_elementaire.zoneoccupee:trect;
var
x:trect;
i:integer;
begin
if traits.count=0 then
   begin
   result.top:=0;
   result.left:=0;
   result.right:=0;
   result.bottom:=0;
   end;
result:=ttrait(traits.items[0]).zoneoccupee;
for i:=1 to traits.count-1 do
    begin
    x:=ttrait(traits.items[i]).zoneoccupee;
    if result.top>x.top then result.top:=x.top;
    if result.left>x.left then result.left:=x.left;
    if result.right<x.right then result.right:=x.right;
    if result.bottom<x.bottom then result.bottom:=x.bottom;
    end;
end;

procedure tgraphe_elementaire.translate(ax,ay:integer);
var i:integer;
begin
for i:=0 to traits.count-1 do ttrait(traits.items[i]).translate(ax,ay);
end;

function tgraphe_elementaire.element(i:integer):ttrait;
begin
if i<traits.count then result:=traits.items[i] else result:=nil;
end;

function tgraphe_elementaire.nombre:integer;
begin
result:=traits.count;
end;

function tgraphe_elementaire.max:integer;
begin
result:=traits.count-1;
end;

destructor tgraphe_elementaire.free;
var i:integer;
begin
for i:=0 to traits.count-1 do ttrait(traits.items[i]).free;
traits.free;
inherited free;
end;


function tnoeud.zoneoccupee:trect;
begin
result.top:=0;
result.left:=0;
result.right:=0;
result.bottom:=0;
end;

procedure tnoeud.dessine(canvas:tcanvas);
begin
end;

procedure tnoeud.translate(ax,ay:integer);
begin
end;

function tnoeud.point_d_ancrage:tpoint;
begin
result.x:=0;
result.y:=0;
end;

function tnoeud.point_de_depart(i:integer):tpoint;
begin
result.x:=0;
result.y:=0;
end;





constructor tsommet.create_point(ax,ay:integer;alibelle:string);
var
x:ptype_sommet;
begin
inherited create;
pos:=-1;
new(x);
x.x:=ax;
x.y:=ay;
if length(alibelle)>0 then
      if (alibelle[1]>='a') and (alibelle[1]<='z') then alibelle[1]:=chr(byte(alibelle[1])-32);
x.libelle:=copy(alibelle,1,2);
points:=tlist.create;
points.add(x);
end;

constructor tsommet.create_ensemble(liste:tlist);
var i:integer;
begin
pos:=-1;
if points<>nil then
   for i:=0 to points.count -1 do
   dispose(ptype_sommet(points.items[i]));
points:=liste;
end;

function tsommet.ajoute_point(ax,ay:integer;alibelle:string):integer;
var
x:ptype_sommet;

begin
if points=nil then points:=tlist.create;
new(x);
x.x:=ax;
x.y:=ay;
x.libelle:=copy(alibelle,1,2);
result:=points.add(x);
end;

function tsommet.retire_point(position:integer):integer;
var
x:ptype_sommet;
begin
if points=nil then exit;
if position>=points.count then exit;
x:=ptype_sommet(points.items[position]);
dispose(x);
points.delete(position);
result:=0;
end;



function tsommet.zoneoccupee:trect;
begin
result:=inherited zoneoccupee;
if (points<>nil) then if points.count>0 then
   begin
   result.top:=ptype_sommet(points.items[0])^.y-(granularite div 2)+1;
   result.left:=ptype_sommet(points.items[0])^.x-(granularite div 2)+1;
   result.bottom:=ptype_sommet(points.items[0])^.y+(points.Count-1)*(granularite div 2)+(granularite div 2)-1;
   result.right:=ptype_sommet(points.items[0])^.x+(granularite div 2)-1;
   end;
end;

procedure tsommet.dessine(canvas:tcanvas);
var i,x,y:integer;
libelle:string[1];
          begin
          if points=nil then exit;
          if points.count=0 then exit;
          with canvas do
               begin
//               brush.color:=clred;
//               brush.style:=bsBDiagonal;
//               fillrect(zoneoccupee);
               font.color:=couleur;
               font.Name:='Arial';
               if taille_point>0 then font.size:=taille_point else font.size:=4;
               if not(sortie_ecran) then font.size:=font.size div 2;
               x:=ptype_sommet(points.items[0])^.x;
               y:=ptype_sommet(points.items[0])^.y;
               for i:=0 to points.count-1 do
                   begin
                   brush.color:=couleur;
                   brush.style:=bssolid;
                   ellipse(x-taille_point,y-taille_point,x+taille_point,y+taille_point);
                   libelle:=copy(ptype_sommet(points.items[i])^.libelle,1,1);
                   brush.style:=bsclear;
                   if fond_blanc then
                   begin
                   font.color:=clwhite;
                   textout(1+x-textwidth(libelle) div 2,1+y-taille_point-textheight(libelle),libelle);
                   textout(1+x-textwidth(libelle) div 2,y-taille_point-textheight(libelle),libelle);
                   textout(1+x-textwidth(libelle) div 2,-1+y-taille_point-textheight(libelle),libelle);
                   textout(-1+x-textwidth(libelle) div 2,1+y-taille_point-textheight(libelle),libelle);
                   textout(-1+x-textwidth(libelle) div 2,y-taille_point-textheight(libelle),libelle);
                   textout(-1+x-textwidth(libelle) div 2,-1+y-taille_point-textheight(libelle),libelle);
                   textout(x-textwidth(libelle) div 2,1+y-taille_point-textheight(libelle),libelle);
                   textout(x-textwidth(libelle) div 2,-1+y-taille_point-textheight(libelle),libelle);
                   font.color:=couleur;
                   end;
                   textout(x-textwidth(libelle) div 2,y-taille_point-textheight(libelle),libelle);
                   y:=y+granularite div 2;
                   end;
               if points.count>1 then ellipse(x-granularite div 4,ptype_sommet(points.items[0])^.y-granularite div 2+taille_point*2,x+granularite div 4,y-taille_point*2);
               end;
          end;

procedure tsommet.translate(ax,ay:integer);
var i:integer;
begin
if points=nil then exit;
for i:=0 to points.count-1 do
    with ptype_sommet(points.items[i])^ do
         begin
         x:=x+ax;
         y:=y+ay;
         end;
end;

function tsommet.point_d_ancrage:tpoint;
var ycentre:integer;
begin
result:=inherited point_d_ancrage;
if points<>nil then
   begin
   if points.count=1 then
      begin
      result.y:=ptype_sommet(points.items[0])^.y;
      result.x:=ptype_sommet(points.items[0])^.x;
      end;
   if points.count>1 then
      begin
      result.x:=ptype_sommet(points.items[0])^.x-granularite div 4;
      ycentre:=ptype_sommet(points.items[0])^.y;
      ycentre:=ycentre*2;
      ycentre:=ycentre+(points.count-1)*(granularite div 2);
      result.y:=ycentre div 2;
      end;
   end;
end;

function tsommet.hauteur:integer;
begin
result:=0;
if points<>nil then
   if points.count>1 then
      result:=(points.count+1)*(granularite div 2);
end;


function tsommet.point_de_depart(i:integer):tpoint;
var ycentre:integer;
x,y:real;
begin
result:=inherited point_de_depart(i);
if points=nil then exit;
if (i>points.count) and (i<>DEPART_GENERAL) then exit;
ycentre:=ptype_sommet(points.items[0])^.y;
result.y:=ycentre;
result.x:=ptype_sommet(points.items[0])^.x;
if points.count=1 then exit;
if points.count>1 then
   begin
   ycentre:=ycentre*2+(points.count-1)*(granularite div 2);
   ycentre:=ycentre div 2;
   end;
result.x:=ptype_sommet(points.items[0])^.x;
if i=DEPART_GENERAL then
   begin
   result.y:=ycentre;
   result.x:=result.x+granularite div 4;
   exit;
   end;
y:=ptype_sommet(points.items[0])^.y+i*(granularite div 2);
x:=(points.count+2);
x:=x*(granularite div 4);
x:=x*(ycentre-y);
x:=x*(ycentre-y);
x:=x/granularite;
x:=sqrt(x);
result.x:=result.x+round(x);
result.y:=round(y);
end;

function tsommet.position(libelle:string):integer;
var i:integer;
begin
libelle:=copy(libelle,1,2);
if length(libelle)>0 then
      if (libelle[1]>='a') and (libelle[1]<='z') then libelle[1]:=chr(byte(libelle[1])-32);
result:=-1;
if points=nil then exit;
for i:=0 to points.count-1 do
    if libelle=ptype_sommet(points.items[i]).libelle then
       begin
       result:=i;
       exit;
       end;
end;

function tsommet.element(i:integer):ptype_sommet;
begin
result:=nil;
if points=nil then exit;
if i>=points.count then exit;
result:=points.items[i];
end;

function tsommet.nombre:integer;
begin
result:=points.count;
end;

function tsommet.max:integer;
begin
result:=points.count-1;
end;

function tsommet.etiquettes:string;
var
i:integer;
begin
result:='';
for i:=0 to points.count-1 do if points.items[i]<>nil then
    result:=result+ptype_sommet(points.items[i]).libelle
end;

destructor tsommet.free;
var i:integer;
begin
if points<>nil then
   for i:=0 to points.count -1 do
   dispose(ptype_sommet(points.items[i]));
points.free;
//inherited free;
end;

constructor ttrait.create(asommet1,asommet2:tsommet;adepart:integer;afleche:boolean);
begin
inherited create;
sommet1:=asommet1;
sommet2:=asommet2;
depart:=adepart;
fleche:=afleche;
end;

function ttrait.zoneoccupee:trect;
var
x:trect;
begin
result:=sommet1.zoneoccupee;
x:=sommet2.zoneoccupee;
if result.top>x.top then result.top:=x.top;
if result.left>x.left then result.left:=x.left;
if result.right<x.right then result.right:=x.right;
if result.bottom<x.bottom then result.bottom:=x.bottom;
end;

procedure ttrait.dessine(canvas:tcanvas);
var
point1,point2:tpoint;
begin
if (sommet1=nil) or (sommet2=nil) then exit;
with canvas do
     begin
     pen.color:=couleur;
     pen.style:=pssolid;
     Pen.mode:=pmcopy;
     brush.color:=couleur;
     brush.style:=bssolid;
     point1:=sommet1.point_de_depart(depart);
     point2:=sommet2.point_d_ancrage;
     if not fleche then
        begin
        moveto(point1.x,point1.y);
        lineto(point2.x,point2.y);
        end
     else dessinefleche(canvas,point1.x,point1.y,point2.x,point2.y,taille_point*3);
     end;
end;

procedure ttrait.translate(ax,ay:integer);
begin
if (sommet1<>nil) then sommet1.translate(ax,ay);
if (sommet2<>nil) then sommet2.translate(ax,ay);
end;

function ttrait.point_d_ancrage:tpoint;
begin
result:=inherited point_d_ancrage;
if (sommet1)<>nil then result:=sommet1.point_d_ancrage;
end;

function ttrait.point_de_depart(i:integer):tpoint;
begin
result:=inherited point_de_depart(i);
if (sommet2)<>nil then result:=sommet2.point_de_depart(i);
end;

function ttrait.reference(libelle:string):byte;
begin
result:=0;
if sommet1<>nil then if sommet1.position(libelle)<>-1 then result:=result+1;
if sommet2<>nil then if sommet2.position(libelle)<>-1 then result:=result+2;
end;

function ttrait.element(i:integer):tsommet;
begin
result:=nil;
if i=0 then if sommet1<>nil then result:=sommet1 else result:=sommet2;
if i=1 then if sommet1<>nil then result:=sommet2;
end;

function ttrait.nombre:integer;
begin
result:=0;
if sommet1<>nil then result:=result+1;
if sommet2<>nil then result:=result+1;
end;

function ttrait.max:integer;
begin
result:=nombre-1;
end;

destructor ttrait.free;
begin
//if somme1<>nil then sommet1.free;
//if sommet2<>nil then sommet2.free;
//inherited free;
end;


end.
